@@ -23,22 +23,17 @@ all_colors = (
 
 @click.command()
 def cli():
-    """This script prints some colors.  If colorama is installed this will
-    also work on Windows.  It will also automatically remove all ANSI
-    styles if data is piped into a file.
+    """This script prints some colors. It will also automatically remove
+    all ANSI styles if data is piped into a file.
 
     Give it a try!
     """
     for color in all_colors:
-        click.echo(click.style("I am colored {}".format(color), fg=color))
+        click.echo(click.style(f"I am colored {color}", fg=color))
     for color in all_colors:
-        click.echo(
-            click.style("I am colored {} and bold".format(color), fg=color, bold=True)
-        )
+        click.echo(click.style(f"I am colored {color} and bold", fg=color, bold=True))
     for color in all_colors:
-        click.echo(
-            click.style("I am reverse colored {}".format(color), fg=color, reverse=True)
-        )
+        click.echo(click.style(f"I am reverse colored {color}", fg=color, reverse=True))
 
     click.echo(click.style("I am blinking", blink=True))
     click.echo(click.style("I am underlined", underline=True))
